/*  
 *  Arduino Line Following Robot
 *  Dev: Michalis Vasilakis // Date: 16/7/2017 
 *  More info can be found at http://www.ardumotive.com
 */ 

//Constants - Connections
const int buzzer = 4;     
const int motorA1 = 5;  //Left Motor   
const int motorA2 = 6;  
const int motorB1 = 10; //Right Motor    
const int motorB2 = 9; 
const int sensor2 = A1; //Left infrared sensor
const int sensor3 = A2; //Middle infrared sensor
const int sensor4 = A3; //Right infrared sensor
const int button = A5;  //On first click: Start / On second click: Stop

//Variables
int leftSensor;
int rightSensor;
int middleSensor;
int buttonState;
boolean start = false;

//For motor speed - pwm
int speedLeft = 235;
int speedRight = 255;

void setup(){
  //Define I/O pins
  pinMode(buzzer, OUTPUT);
  pinMode(motorA1, OUTPUT);
  pinMode(motorA2, OUTPUT);
  pinMode(motorB1, OUTPUT);
  pinMode(motorB2, OUTPUT);
  Serial.begin(9600);

}

void loop(){
middleSensor = digitalRead(sensor3);
    Serial.print(middleSensor);

  buttonState = digitalRead(button);   //Read button state 
  if (buttonState==HIGH){
    while(buttonState==HIGH){          //Wait until button is unpressed!
      buttonState = digitalRead(button);
      delay(10);
    }
    Serial.println("Button");
    //Init condition: robot is stopped, on first press: robot is moving, on second press: robot is stopped.
    if (!start){
      start = true;
    }
    else{
      start = false;
    }
    delay(250);
  }

  if (!start){
    stop(); // Stop the robot!
  }
  else{
    leftSensor = digitalRead(sensor2);
    middleSensor = digitalRead(sensor3);
    rightSensor = digitalRead(sensor4);
    //Follow black line: Black 1 / White 0
    if (leftSensor==HIGH && rightSensor==HIGH && middleSensor==LOW){ // Black line is in the middle - Sensors are looking "white", so move forward
      forward();
      noTone(buzzer);
    }
    else if (leftSensor==LOW && rightSensor==HIGH && (middleSensor==LOW || middleSensor==HIGH)){ //Left sensor detect black line, turn right to correct robot route
      right();
      tone(buzzer,500);
    }
    else if (leftSensor==HIGH && rightSensor==LOW && (middleSensor==LOW || middleSensor==HIGH)){ //Right sensor detect black line, turn left to correct robot route
      left();
      tone(buzzer,500);
    }
    else if (leftSensor==LOW && rightSensor==LOW && middleSensor==LOW){ //Finish line detected! Stop the robot 
      stop();
      tone(buzzer,800,1000);
      start = false;
    }
   /* else if (leftSensor==HIGH && rightSensor==HIGH && middleSensor==HIGH){ // No line detected? Well, do something or stop it.
      stop();
      tone(buzzer,200,1000);
      start = false;
    }*/
    
  }
}

//Moving functions
void forward(){
  analogWrite(motorA1, speedLeft);
  analogWrite(motorA2, 0);
  analogWrite(motorB1, speedRight);
  analogWrite(motorB2, 0); 
}

void backward(){
  analogWrite(motorA1, 0);
  analogWrite(motorA2, speedLeft);
  analogWrite(motorB1, 0);
  analogWrite(motorB2, speedRight);
}

void left(){
  analogWrite(motorA1, speedLeft);
  analogWrite(motorA2, 0);
  analogWrite(motorB1, 0);
  analogWrite(motorB2, speedRight);
}

void right(){
  analogWrite(motorA1, 0);
  analogWrite(motorA2, speedLeft);
  analogWrite(motorB1, speedRight);
  analogWrite(motorB2, 0); 
}

void stop(){
  analogWrite(motorA1, 0);
  analogWrite(motorA2, 0);
  analogWrite(motorB1, 0);
  analogWrite(motorB2, 0);
}
